@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Davide Alberani <da@erlug.linux.it>
+# Copyright 2004-2020 Davide Alberani <da@erlug.linux.it>
 #                2008 H. Turgut Uyar <uyar@tekir.org>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -26,14 +26,15 @@ or "html" (this is the default).
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import logging
 import socket
 import ssl
 from codecs import lookup
+import warnings
 
 from imdb import PY2
 from imdb import IMDbBase
 from imdb.utils import analyze_title
+from imdb.parser.http.logging import logger
 from imdb._exceptions import IMDbDataAccessError, IMDbParserError
 
 from . import (
@@ -41,11 +42,13 @@ from . import (
     movieParser,
     personParser,
     searchMovieParser,
+    searchMovieAdvancedParser,
     searchPersonParser,
     searchCompanyParser,
-    searchKeywordParser
+    searchKeywordParser,
+    topBottomParser,
+    listParser
 )
-from . import topBottomParser
 
 if PY2:
     from urllib import quote_plus
@@ -55,7 +58,7 @@ else:
     from urllib.request import HTTPSHandler, ProxyHandler, build_opener
 
 # Logger for miscellaneous functions.
-_aux_logger = logging.getLogger('imdbpy.parser.http.aux')
+_aux_logger = logger.getChild('aux')
 
 
 class _ModuleProxy:
@@ -88,16 +91,6 @@ class _ModuleProxy:
         return getattr(_sm, name)
 
 
-# The cookies for the "adult" search.
-# Please don't mess with these account.
-_cookie_id = 'BCYmoyqSm2WglmOzG-SrFWSvVpxsTZOB0qEOOqmAwCBxCbaNgKOxd0DTKzUvt7t04Pya5gV2tUrpDmYxrc1Dr54DQj2UX' \
-    'I7QI35__M5-HI2KrbOI3PjDz6M-_U3HG8topMfN64R24tmBixoZhMYXVaEc556lf0Z4gQNJVYRANXvwytP5v1lpfeToRlu9aVJwN4kT'
-_cookie_uu = 'BCYquDS8Y2i8R1pJxS4nB77YrhjHHXeOea2Xl9KtZvE6RZKVfMvzTGU4Vl5-yxfPbgRSiFJasyf-hhPuVvXyaHlfeBjNl' \
-    'bFT8hz2HzFFkQ_SxKxq05J51gi7Fv4SaAws1M-i7zmQ1TRunfJqCVIYqPwIs2NO7s4_YDH2ZoISVGLgca8OY2K58HychOZB1oRWHVe' \
-    'AJNhLJMrCWJBuGRLCNnQK5X9tA0dPPntr2Ussy0ouul-N1GQz-8y5vda3JJ_C6xkwmHcA6JrOdOFO_HqMWjVSXuxGEdrXC919JM9H0' \
-    'vooVvKeVgAEJnTh2GiVlUJUoH3c'
-
-
 class _FakeURLOpener(object):
     """Fake URLOpener object, used to return empty strings instead of
     errors.
@@ -159,7 +152,7 @@ class IMDbHTTPSHandler(HTTPSHandler, object):
 
 class IMDbURLopener:
     """Fetch web pages and handle errors."""
-    _logger = logging.getLogger('imdbpy.parser.http.urlopener')
+    _logger = logger.getChild('urlopener')
 
     def __init__(self, *args, **kwargs):
         self._last_url = ''
@@ -170,10 +163,6 @@ class IMDbURLopener:
             self.del_header(header)
         self.set_header('User-Agent', 'Mozilla/5.0')
         self.set_header('Accept-Language', 'en-us,en;q=0.5')
-        # XXX: This class is used also to perform "Exact Primary
-        #      [Title|Name]" searches, and so by default the cookie is set.
-        c_header = 'uu=%s; id=%s' % (_cookie_uu, _cookie_id)
-        self.set_header('Cookie', c_header)
 
     def get_proxy(self):
         """Return the used proxy, or an empty string."""
@@ -278,7 +267,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
     """The class used to access IMDb's data through the web."""
 
     accessSystem = 'http'
-    _http_logger = logging.getLogger('imdbpy.parser.http')
+    _http_logger = logger
 
     def __init__(self, adultSearch=True, proxy=-1, cookie_id=-1,
                  timeout=30, cookie_uu=None, *arguments, **keywords):
@@ -288,18 +277,13 @@ class IMDbHTTPAccessSystem(IMDbBase):
         self._getRefs = True
         self._mdparse = False
         self.set_timeout(timeout)
-        self.do_adult_search(adultSearch)
-        if cookie_id != -1:
-            if cookie_id is None:
-                self.del_cookies()
-            elif cookie_uu is not None:
-                self.set_cookies(cookie_id, cookie_uu)
         if proxy != -1:
             self.set_proxy(proxy)
         _def = {'_modFunct': self._defModFunct, '_as': self.accessSystem}
 
         # Proxy objects.
         self.smProxy = _ModuleProxy(searchMovieParser, defaultKeys=_def)
+        self.smaProxy = _ModuleProxy(searchMovieAdvancedParser, defaultKeys=_def)
         self.spProxy = _ModuleProxy(searchPersonParser, defaultKeys=_def)
         self.scompProxy = _ModuleProxy(searchCompanyParser, defaultKeys=_def)
         self.skProxy = _ModuleProxy(searchKeywordParser, defaultKeys=_def)
@@ -307,6 +291,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
         self.pProxy = _ModuleProxy(personParser, defaultKeys=_def)
         self.compProxy = _ModuleProxy(companyParser, defaultKeys=_def)
         self.topBottomProxy = _ModuleProxy(topBottomParser, defaultKeys=_def)
+        self.listProxy = _ModuleProxy(listParser, defaultKeys=_def)
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
@@ -373,40 +358,24 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def set_cookies(self, cookie_id, cookie_uu):
         """Set a cookie to access an IMDb's account."""
-        c_header = 'id=%s; uu=%s' % (cookie_id, cookie_uu)
-        self.urlOpener.set_header('Cookie', c_header)
+        warnings.warn("set_cookies has been deprecated")
 
     def del_cookies(self):
         """Remove the used cookie."""
-        self.urlOpener.del_header('Cookie')
+        warnings.warn("del_cookies has been deprecated")
 
     def do_adult_search(self, doAdult,
-                        cookie_id=_cookie_id, cookie_uu=_cookie_uu):
+                        cookie_id=None, cookie_uu=None):
         """If doAdult is true, 'adult' movies are included in the
         search results; cookie_id and cookie_uu are optional
         parameters to select a specific account (see your cookie
         or cookies.txt file."""
-        if doAdult:
-            self.set_cookies(cookie_id, cookie_uu)
-        else:
-            self.urlOpener.del_header('Cookie')
+        return
 
     def _retrieve(self, url, size=-1, _noCookies=False):
         """Retrieve the given URL."""
-        # print url
-        _cookies = None
-        # XXX: quite obscene, but in some very limited
-        #      cases (/ttXXXXXXX/epdate) if the cookies
-        #      are set, a 500 error is returned.
-        if _noCookies:
-            _cookies = self.urlOpener.get_header('Cookie')
-            self.del_cookies()
         self._http_logger.debug('fetching url %s (size: %d)', url, size)
-        try:
-            ret = self.urlOpener.retrieve_unicode(url, size=size)
-        finally:
-            if _noCookies and _cookies:
-                self.urlOpener.set_header('Cookie', _cookies)
+        ret = self.urlOpener.retrieve_unicode(url, size=size)
         if PY2 and isinstance(ret, str):
             ret = ret.decode('utf-8')
         return ret
@@ -438,6 +407,42 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._get_search_content('tt', title, results)
         return self.smProxy.search_movie_parser.parse(cont, results=results)['data']
 
+    def _get_list_content(self, list_):
+        """Retrieve a list by it's id"""
+        if list_.startswith('ls'):
+            imdbUrl = self.urls['movie_list'] + list_
+        else:
+            warnings.warn("list type not recognized make sure it starts with 'ls'")
+            return
+        return self._retrieve(url=imdbUrl)
+
+    def _get_movie_list(self, list_, results):
+        cont = self._get_list_content(list_)
+        return self.listProxy.list_parser.parse(cont, results=results)['data']
+
+    def _get_search_movie_advanced_content(self, title=None, adult=None, results=None,
+                                           sort=None, sort_dir=None):
+        """Retrieve the web page for a given search.
+        results is the maximum number of results to be retrieved."""
+        criteria = {}
+        if title is not None:
+            criteria['title'] = quote_plus(title, safe='')
+        if adult:
+            criteria['adult'] = 'include'
+        if results is not None:
+            criteria['count'] = str(results)
+        if sort is not None:
+            criteria['sort'] = sort
+            if sort_dir is not None:
+                criteria['sort'] = sort + ',' + sort_dir
+        params = '&'.join(['%s=%s' % (k, v) for k, v in criteria.items()])
+        return self._retrieve(self.urls['search_movie_advanced'] % params)
+
+    def _search_movie_advanced(self, title=None, adult=None, results=None, sort=None, sort_dir=None):
+        cont = self._get_search_movie_advanced_content(title=title, adult=adult, results=results,
+                                                       sort=sort, sort_dir=sort_dir)
+        return self.smaProxy.search_movie_advanced_parser.parse(cont, results=results)['data']
+
     def _search_episode(self, title, results):
         t_dict = analyze_title(title)
         if t_dict['kind'] == 'episode':
@@ -449,6 +454,14 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(self.urls['movie_main'] % movieID + 'reference')
         return self.mProxy.movie_parser.parse(cont, mdparse=self._mdparse)
 
+    def get_movie_recommendations(self, movieID):
+        cont = self._retrieve(self.urls['movie_main'] % movieID)
+        r = {'info sets': ('recommendations',), 'data': {}}
+        ret = self.mProxy.movie_parser.parse(cont, mdparse=self._mdparse)
+        if 'data' in ret and 'recommendations' in ret['data'] and ret['data']['recommendations']:
+            r['data']['recommendations'] = ret['data']['recommendations']
+        return r
+
     def get_movie_full_credits(self, movieID):
         cont = self._retrieve(self.urls['movie_main'] % movieID + 'fullcredits')
         return self.mProxy.full_credits_parser.parse(cont)
@@ -586,31 +599,36 @@ class IMDbHTTPAccessSystem(IMDbBase):
             del data_d['data']['_seasons']
         return data_d
 
-    def get_movie_episodes(self, movieID):
+    def get_movie_episodes(self, movieID, season_nums='all'):
         cont = self._retrieve(self.urls['movie_main'] % movieID + 'episodes')
-        data_d = self.mProxy.season_episodes_parser.parse(cont)
-        if not data_d and 'data' in data_d:
+        temp_d = self.mProxy.season_episodes_parser.parse(cont)
+        if not isinstance(season_nums, list):
+            if season_nums != 'all':
+                season_nums = [season_nums]
+        if not temp_d and 'data' in temp_d:
             return {}
-        _current_season = data_d['data'].get('_current_season', '')
-        _seasons = data_d['data'].get('_seasons') or []
-        data_d = self._purge_seasons_data(data_d)
-        data_d['data'].setdefault('episodes', {})
 
-        nr_eps = len(data_d['data']['episodes'].get(_current_season) or [])
+        _seasons = temp_d['data'].get('_seasons') or []
+
+        nr_eps = 0
+        data_d = dict()
 
         for season in _seasons:
-            if season == _current_season:
+            if season_nums != 'all' and season not in season_nums:
                 continue
-            other_cont = self._retrieve(
+            cont = self._retrieve(
                 self.urls['movie_main'] % movieID + 'episodes?season=' + str(season)
             )
-            other_d = self.mProxy.season_episodes_parser.parse(other_cont)
+            other_d = self.mProxy.season_episodes_parser.parse(cont)
             other_d = self._purge_seasons_data(other_d)
             other_d['data'].setdefault('episodes', {})
             if not (other_d and other_d['data'] and other_d['data']['episodes'][season]):
                 continue
             nr_eps += len(other_d['data']['episodes'].get(season) or [])
-            data_d['data']['episodes'][season] = other_d['data']['episodes'][season]
+            if data_d:
+                data_d['data']['episodes'][season] = other_d['data']['episodes'][season]
+            else:
+                data_d = other_d
         data_d['data']['number of episodes'] = nr_eps
         return data_d
 
@@ -700,9 +718,12 @@ class IMDbHTTPAccessSystem(IMDbBase):
             return []
         return self.skProxy.search_keyword_parser.parse(cont, results=results)['data']
 
-    def _get_keyword(self, keyword, results):
+    def _get_keyword(self, keyword, results, page=None):
         try:
-            cont = self._retrieve(self.urls['keyword_main'] % keyword)
+            url = self.urls['keyword_main'] % keyword
+            if page is not None:
+                url = url + "&page=" + str(page)
+            cont = self._retrieve(url)
         except IMDbDataAccessError:
             self._http_logger.warn('unable to get keyword %s', keyword,
                                    exc_info=True)
@@ -716,6 +737,18 @@ class IMDbHTTPAccessSystem(IMDbBase):
         elif kind == 'bottom':
             parser = self.topBottomProxy.bottom100_parser
             url = self.urls['bottom100']
+        elif kind == 'moviemeter':
+            parser = self.topBottomProxy.moviemeter100_parser
+            url = self.urls['moviemeter100']
+        elif kind == 'toptv':
+            parser = self.topBottomProxy.toptv250_parser
+            url = self.urls['toptv250']
+        elif kind == 'tvmeter':
+            parser = self.topBottomProxy.tvmeter100_parser
+            url = self.urls['tvmeter100']
+        elif kind == 'topindian250':
+            parser = self.topBottomProxy.topindian250_parser
+            url = self.urls['topindian250']
         else:
             return []
         cont = self._retrieve(url)
